"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCognito = exports.Cognito = exports.CognitoProvider = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const amazon_cognito_identity_js_1 = require("amazon-cognito-identity-js");
const CognitoContext = (0, react_1.createContext)(undefined);
function CognitoProvider(props) {
    const [auth, setAuth] = (0, react_1.useState)({ cognito: props.value });
    (0, react_1.useEffect)(() => {
        props.value.onChange((cognito) => setAuth({
            cognito,
        }));
        props.value.init()?.catch();
    }, []);
    return ((0, jsx_runtime_1.jsx)(CognitoContext.Provider, { value: auth, children: props.children }));
}
exports.CognitoProvider = CognitoProvider;
class Cognito {
    pool;
    callbacks = [];
    _session;
    _isInitializing = true;
    get isInitializing() {
        return this._isInitializing;
    }
    set isInitializing(val) {
        this._isInitializing = val;
        this.trigger();
    }
    get session() {
        return this._session;
    }
    set session(session) {
        this._session = session;
        this.trigger();
    }
    get user() {
        return this.pool.getCurrentUser();
    }
    constructor(opts) {
        this.pool = new amazon_cognito_identity_js_1.CognitoUserPool({
            UserPoolId: opts.UserPoolId,
            ClientId: opts.ClientId,
        });
    }
    onChange(cb) {
        this.callbacks.push(cb);
    }
    trigger() {
        for (const cb of this.callbacks) {
            cb(this);
        }
    }
    init() {
        if (!this.user) {
            this.isInitializing = false;
            return Promise.resolve(undefined);
        }
        return new Promise((resolve) => {
            this.user.getSession((err, session) => {
                if (err) {
                    this.user.signOut();
                    resolve(undefined);
                    this.isInitializing = false;
                    return;
                }
                this.session = session;
                resolve(session);
                this.isInitializing = false;
            });
        });
    }
    async login(email, password) {
        const user = new amazon_cognito_identity_js_1.CognitoUser({
            Username: email,
            Pool: this.pool,
        });
        const authDetails = new amazon_cognito_identity_js_1.AuthenticationDetails({
            Username: email,
            Password: password,
        });
        return new Promise((resolve, reject) => {
            user.authenticateUser(authDetails, {
                onSuccess: (result) => {
                    this.session = result;
                    this.trigger();
                    resolve(result);
                },
                onFailure: (err) => {
                    reject(err);
                },
            });
        });
    }
    async register(email, password, attributes = []) {
        return new Promise((resolve, reject) => {
            this.pool.signUp(email, password, [
                ...attributes,
                new amazon_cognito_identity_js_1.CognitoUserAttribute({
                    Name: "email",
                    Value: email,
                }),
            ], [], (err, resp) => {
                if (err) {
                    reject(err);
                    return;
                }
                this.trigger();
                resolve(resp);
            });
        });
    }
    async confirm(username, code) {
        const user = new amazon_cognito_identity_js_1.CognitoUser({
            Username: username,
            Pool: this.pool,
        });
        return new Promise((resolve, reject) => {
            user.confirmRegistration(code, true, (err, result) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(result);
            });
        });
    }
    async resend(username) {
        const user = new amazon_cognito_identity_js_1.CognitoUser({
            Username: username,
            Pool: this.pool,
        });
        return new Promise((resolve, reject) => {
            user.resendConfirmationCode((err, result) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(result);
            });
        });
    }
}
exports.Cognito = Cognito;
function useCognito() {
    const auth = (0, react_1.useContext)(CognitoContext);
    return auth.cognito;
}
exports.useCognito = useCognito;
